==> dxde_1.40.6-1.tar.gz: BUILD/src/smtp/wxEmailMessage.cpp <==
/***************************************************************
 * Name:      wxEmailMessage.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 *            Eran Ifrah
 * Copyright: 2017
 * License:   GPL3
 **************************************************************/

#include "wxEmailMessage.h"
#include "wxEmailMessageID.h"
#include <wx/datetime.h>
#include <wx/ffile.h>
#include <wx/base64.h>
#include <wx/msgdlg.h>

#define BOUNDRY_LINE "pj+EhsWuSQJxx7pr"

wxEmailMessage::wxEmailMessage()
    : m_aschar(NULL)
    , m_pos(0)
    , m_len(0)
{
}

wxEmailMessage::~wxEmailMessage()
{
    if(m_aschar) {
        free(m_aschar);
    }
    m_aschar = NULL;
}

wxString wxEmailMessage::GenerateID()
{
    static time_t starupTime = time(NULL);
    time_t curtime = time(NULL);

    wxString s;
    s << starupTime << "." << curtime;
    wxEmailMessageID id(s);
    s = id.Encrypt();

    s.Prepend("<");
    s << "@wxmailer>";
    return s;
}

wxString wxEmailMessage::PayLoad() const
{
    char month[][4] = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
    struct tm* timeinfo = wxDateTime::GetTmNow();
    wxString payload;
    payload << wxString::Format("Date: %d %s %d %d:%d:%d", timeinfo->tm_mday,
                                month[timeinfo->tm_mon], timeinfo->tm_year+1900, timeinfo->tm_hour,
                                timeinfo->tm_min, timeinfo->tm_sec) << "\r\n";
    payload << "To: " << GetTo() << "\r\n";
    payload << "From: " << GetFrom() << "\r\n";
    payload << "Message-ID: " << GenerateID() << "\r\n";
    payload << "Subject: =?UTF-8?B?" << ::wxBase64Encode(GetSubject().mb_str(wxConvUTF8).data(), GetSubject().size()) << "?=\r\n";
    payload << "Reply-To: " << GetFromEmail() << "\r\n";

    // Sending attachment
    payload << "MIME-Version: 1.0\r\n";
    payload << "Content-Type: multipart/mixed; boundary=\"" << BOUNDRY_LINE << "\"\r\n";
    payload << "\r\n";

    // Message body
    if(!GetMessage().IsEmpty()) {
        payload << "--" << BOUNDRY_LINE << "\r\n";
        payload << "Content-Type: text/plain; charset=\"utf-8\"\r\n";
        payload << "Content-Transfer-Encoding: 8bit\r\n";
        payload << "\r\n";
        payload << GetMessage() << "\r\n";
    }

    if(!m_attachements.IsEmpty()) {
        for(size_t i = 0; i < m_attachements.size(); ++i) {
            DoAddAttachment(m_attachements.Item(i), payload);
        }
    }

    payload << "--" << BOUNDRY_LINE << "--\r\n";
    return payload;
}

wxString wxEmailMessage::GetFromEmail() const
{
    if(m_from.Contains("<"))
    {
        return m_from.AfterFirst('<').Before('>');
    }
    return m_from;
}

wxString wxEmailMessage::GetToEmail() const
{
    if(m_to.Contains("<"))
    {
        return m_to.AfterFirst('<').Before('>');
    }
    return m_to;
}

void wxEmailMessage::Finalize()
{
    m_payload = PayLoad();
    const char* p = m_payload.mb_str(wxConvUTF8).data();
    m_aschar = (char*)malloc(strlen(p) + 1);
    strcpy(m_aschar, p);
    m_len = strlen(m_aschar);
    m_pos = 0;
}

void wxEmailMessage::DoAddAttachment(const wxString& filename, wxString& payload) const
{
    wxString base64Attachment;
    wxFileName fn(filename);

    if(!fn.IsOk() || !fn.Exists()) return;

    FILE* fp = fopen(fn.GetFullPath().mb_str(wxConvUTF8).data(), "rb");
    if(fp) {
        fseek(fp, 0, SEEK_END);
        size_t len = ftell(fp);
        fseek(fp, 0, SEEK_SET);

        char* buffer = new char[len];
        fread(buffer, 1, len, fp);
        fclose(fp);
        base64Attachment = ::wxBase64Encode(buffer, len);
        wxDELETEA(buffer);
    }

    if(!base64Attachment.IsEmpty()) {
        wxString encodedFilename = "=?UTF-8?B?";
        encodedFilename << ::wxBase64Encode(fn.GetFullName().mb_str(wxConvUTF8).data(), fn.GetFullName().size()) << "?=";
        payload << "\r\n--" << BOUNDRY_LINE << "\r\n";
        payload << "Content-Type: application/octet-stream; name=\"" << encodedFilename << "\"\r\n";
        payload << "Content-Transfer-Encoding: base64 \r\n";
        payload << "Content-Disposition: attachement; filename=\"" << encodedFilename << "\"\r\n";
        payload << "\r\n";
        // Split the content to 72 chars per line
        while(!base64Attachment.IsEmpty()) {
            size_t bytes = (base64Attachment.length() >= 72) ? 72 : base64Attachment.length();
            wxString line = base64Attachment.Mid(0, bytes);
            payload << line << "\r\n";
            base64Attachment = base64Attachment.Mid(bytes);
        }
    }
}

==> dxde_1.40.12-1.tar.gz: BUILD/src/smtp/wxEmailMessage.cpp <==
/***************************************************************
 * Name:      wxEmailMessage.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 *            Eran Ifrah
 * Copyright: 2017
 * License:   GPL3
 **************************************************************/

#include "wxEmailMessage.h"
#include "wxEmailMessageID.h"
#include <wx/datetime.h>
#include <wx/ffile.h>
#include <wx/base64.h>
#include <wx/msgdlg.h>

#define BOUNDRY_LINE "pj+EhsWuSQJxx7pr"

wxEmailMessage::wxEmailMessage()
    : m_aschar(nullptr)
    , m_pos(0)
    , m_len(0)
{
}

wxEmailMessage::~wxEmailMessage()
{
    if(m_aschar) {
        free(m_aschar);
    }
    m_aschar = nullptr;
}

wxString wxEmailMessage::GenerateID()
{
    static time_t starupTime = time(nullptr);
    time_t curtime = time(nullptr);

    wxString s;
    s << starupTime << "." << curtime;
    wxEmailMessageID id(s);
    s = id.Encrypt();

    s.Prepend("<");
    s << "@wxmailer>";
    return s;
}

wxString wxEmailMessage::PayLoad() const
{
    char month[][4] = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
    struct tm* timeinfo = wxDateTime::GetTmNow();
    wxString payload;
    payload << wxString::Format("Date: %d %s %d %d:%d:%d", timeinfo->tm_mday,
                                month[timeinfo->tm_mon], timeinfo->tm_year+1900, timeinfo->tm_hour,
                                timeinfo->tm_min, timeinfo->tm_sec) << "\r\n";
    payload << "To: " << GetTo() << "\r\n";
    payload << "From: " << GetFrom() << "\r\n";
    payload << "Message-ID: " << GenerateID() << "\r\n";
    payload << "Subject: =?UTF-8?B?" << ::wxBase64Encode(GetSubject().mb_str(wxConvUTF8).data(), strlen(GetSubject().mb_str(wxConvUTF8).data())) << "?=\r\n";
    payload << "Reply-To: " << GetFromEmail() << "\r\n";

    // Sending attachment
    payload << "MIME-Version: 1.0\r\n";
    payload << "Content-Type: multipart/mixed; boundary=\"" << BOUNDRY_LINE << "\"\r\n";
    payload << "\r\n";

    // Message body
    if(!GetMessage().IsEmpty()) {
        payload << "--" << BOUNDRY_LINE << "\r\n";
        payload << "Content-Type: text/plain; charset=\"utf-8\"\r\n";
        payload << "Content-Transfer-Encoding: 8bit\r\n";
        payload << "\r\n";
        payload << GetMessage() << "\r\n";
    }

    if(!m_attachements.IsEmpty()) {
        for(size_t i = 0; i < m_attachements.size(); ++i) {
            DoAddAttachment(m_attachements.Item(i), payload);
        }
    }

    payload << "--" << BOUNDRY_LINE << "--\r\n";
    return payload;
}

wxString wxEmailMessage::GetFromEmail() const
{
    if(m_from.Contains("<"))
    {
        return m_from.AfterFirst('<').Before('>');
    }
    return m_from;
}

wxString wxEmailMessage::GetToEmail() const
{
    if(m_to.Contains("<"))
    {
        return m_to.AfterFirst('<').Before('>');
    }
    return m_to;
}

void wxEmailMessage::Finalize()
{
    m_payload = PayLoad();
    const char* p = m_payload.mb_str(wxConvUTF8).data();
    m_aschar = static_cast<char*>(malloc(strlen(p) + 1));
    strcpy(m_aschar, p);
    m_len = strlen(m_aschar);
    m_pos = 0;
}

void wxEmailMessage::DoAddAttachment(const wxString& filename, wxString& payload) const
{
    wxString base64Attachment;
    wxFileName fn(filename);

    if(!fn.IsOk() || !fn.Exists()) return;

    FILE* fp = fopen(fn.GetFullPath().mb_str(wxConvUTF8).data(), "rb");
    if(fp) {
        fseek(fp, 0, SEEK_END);
        size_t len = static_cast<size_t>(ftell(fp));
        fseek(fp, 0, SEEK_SET);

        char* buffer = new char[len];
        fread(buffer, 1, len, fp);
        fclose(fp);
        base64Attachment = ::wxBase64Encode(buffer, len);
        wxDELETEA(buffer);
    }

    if(!base64Attachment.IsEmpty()) {
        wxString encodedFilename = "=?UTF-8?B?";
        encodedFilename << ::wxBase64Encode(fn.GetFullName().mb_str(wxConvUTF8).data(), fn.GetFullName().size()) << "?=";
        payload << "\r\n--" << BOUNDRY_LINE << "\r\n";
        payload << "Content-Type: application/octet-stream; name=\"" << encodedFilename << "\"\r\n";
        payload << "Content-Transfer-Encoding: base64 \r\n";
        payload << "Content-Disposition: attachement; filename=\"" << encodedFilename << "\"\r\n";
        payload << "\r\n";
        // Split the content to 72 chars per line
        while(!base64Attachment.IsEmpty()) {
            size_t bytes = (base64Attachment.length() >= 72) ? 72 : base64Attachment.length();
            wxString line = base64Attachment.Mid(0, bytes);
            payload << line << "\r\n";
            base64Attachment = base64Attachment.Mid(bytes);
        }
    }
}
